--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of DKPmon.

    DKPmon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DKPmon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DKPmon; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]

local dewdrop = AceLibrary("Dewdrop-2.0")

local dewOptions = {}

--[[
    DKPmon:InitAwardFields()
    Description:
        Initializes the data table: self.awards
    Input:
        None
    Output:
        None
]]
function DKPmon:InitAwardFields()
   self.awards = {}
   self.awards.raid = {}
   self.awards.standby = {}
   self.awards.ndbselected = 0
   self.awards.dblist = nil
   self.awards.boss = nil
   self.awards.bonus = { points = 0, pool = self.poolnames[0] }
   self.awards.selectedpool = 0
   self.GUI.awardUI = {}
end

function DKPmon:CountRaidMembers()
    local n, v, nraiders
    nraiders = 0
    for n,v in pairs(self.awards.raid) do
        nraiders = nraiders + 1
    end
    return nraiders
end

function DKPmon:CountStandbyMembers()
    local n, v, nstandby
    nstandby = 0
    for n,v in pairs(self.awards.standby) do
        nstandby = nstandby + 1
    end
    return nstandby
end

function DKPmon:CountDBSelected()
    local ndbselected = 0
    if self.awards.dblist ~= nil then
        for n,v in pairs(self.awards.dblist) do
            if (v.onlist) then
                ndbselected = ndbselected + 1
            end
        end
    end
    return ndbselected
end

local function BuildMemberlistOptionsTable(options, memtab, headerStr)
   options.args.header = {}
   options.args.header.type = "header"
   options.args.header.name = headerStr
   options.args.header.order = 1
   local name, value
   for name, value in pairs(memtab) do
      local n = name
      local v = value
      options.args[name] = {}
      local tab = options.args[name]
      tab.type = 'toggle'
      if (value.bidname == name) then
        tab.name = string.format("|cff%s%s|r", Bidder.classInfo.classHex[value.senderclass], name)
      else
        tab.name = string.format("|cff%s%s|r[|cff%s%s|r]", 
                                    Bidder.classInfo.classHex[value.senderclass], name, 
                                    Bidder.classInfo.classHex[DKPmon.bidderClasses[value.bidname]], value.bidname)
      end
      tab.desc = string.format("Include/exclude %s in awarding points.", name)
      tab.get = function()
                  return memtab[n].onlist
                end
      tab.set = function(v)
                  memtab[n].onlist = v
                end
      tab.order = string.byte(name, 1) -- the numeric code of the character name's first letter.
                                       -- Results in a list sorted by first letter of the player's names
   end

end
--[[
    DKPmon:PresentRaidList()
    Description:
        Pops up the tooltip to display the list of players in the raid.
    Input:
        None
    Output:
        None
]]
function DKPmon:PresentRaidList()
--   self:Print("Display raid list")
   self:TableDelete(dewOptions) -- Recursively reclaim the table
   dewOptions = { type = "group", args = {} }
   BuildMemberlistOptionsTable(dewOptions, self.awards.raid, "List of raid members")
   dewdrop:Open(self.GUI.awardUI.frame, 'children', dewOptions, 'point', "TOP", 'relativePoint', "TOP")
end

--[[
    DKPmon:PresentStandbyList()
    Description:
        Pops up the tooltip to display the list of players on standby
    Input:
        None
    Output:
        None
]]
function DKPmon:PresentStandbyList()
   --self:Print("Display standby list")
   self:TableDelete(dewOptions) -- Recursively reclaim the table
   dewOptions = { type = "group", args = {} }
   BuildMemberlistOptionsTable(dewOptions, self.awards.standby, "List of standby members")

   dewdrop:Open(self.GUI.awardUI.frame, 'children', dewOptions, 'point', "TOP", 'relativePoint', "TOP")
end

local function BuildBossTable(bosstab, pos)
   local tab = {}
   local name, t
   for name, t in pairs(bosstab) do
      local label = string.format("%d", pos)
      local n = name
      tab[label] = {}
      if bosstab[name][0] == nil then -- Actual boss names will have a [0] table, the categories will not.
         -- Build a group.
         tab[label].type = 'group'
         tab[label].name = name
         tab[label].desc = name
         tab[label].args = BuildBossTable(t, 100)
         tab[label].order = string.byte(name, 1)--pos
      else
         
         tab[label].type = 'execute'
         tab[label].name = name
         tab[label].desc = string.format("Select boss %s", name)
         tab[label].order = string.byte(name, 1)--lab
         tab[label].func = function() 
                    DKPmon.awards.boss = n
                    DKPmon:RefreshAwardUI();
                    dewdrop:Close(1)
                   end
      end
      pos = pos + 1
   end
   return tab
end
--[[
    DKPmon:PresentBossList()
    Description:
        Pops up the tooltip to display the list of bosses from which to select one to award points for.
    Input:
        None
    Output:
        None
]]
function DKPmon:PresentBossList()
   --self:Print("Display boss list")
   self:TableDelete(dewOptions) -- Recursively reclaim the table
   dewOptions = { type = "group" }
   dewOptions.args = BuildBossTable(self.bossvalues, 100)
   dewOptions.args.header = {} 
   dewOptions.args.header.type = "header"
   dewOptions.args.header.name = "List of Bosses"
   dewOptions.args.header.order = 1
   
   --BuildBossTable(self.bossvalues, dewOptions.args, 100)

   dewdrop:Open(self.GUI.awardUI.frame, 'children', dewOptions, 'point', "TOP", 'relativePoint', "TOP")
end

--[[
    DKPmon:PresentDBNameSelectList()
    Description:
        Called when the user clicks in the text field for selecting players from the database to award points to.
        Pops up a dewdrop dialog from which names can be selected for awarding
    Input:
        None
    Output:
        None
]]
function DKPmon:PresentDBNameSelectList()
    -- First, we populate the list of players in the database if we haven't already
    if self.awards.dblist == nil then
        self.awards.dblist = {}
        local name, nametab
        for name, nametab in pairs(self.db.realm.points) do
            self.awards.dblist[name] = { bidname = name, onlist = false, class = nametab.info.class }
        end
    end
    
    -- Build the dewOptions table
    self:TableDelete(dewOptions)
    dewOptions = { type = "group", args = {} }
    dewOptions.args.header = {}
    dewOptions.args.header.type = "header"
    dewOptions.args.header.name = "List of players in the database"
    dewOptions.args.header.order = 1
    
    local name, nametab
    local memtab = self.awards.dblist
    for name, nametab in pairs(self.awards.dblist) do
        local subtab = string.sub(name, 1, 1)
        if dewOptions.args[subtab] == nil then
            dewOptions.args[subtab] = {}
            -- Build a group.
            dewOptions.args[subtab].type = 'group'
            dewOptions.args[subtab].name = subtab
            dewOptions.args[subtab].desc = "Characters starting with the letter "..subtab
            dewOptions.args[subtab].args = {}
            dewOptions.args[subtab].order = string.byte(subtab)
        end
        local n = name
        dewOptions.args[subtab].args[name] = {}
        local tab = dewOptions.args[subtab].args[name]
        tab.type = 'toggle'
        local hexclr = Bidder.classInfo.classHex[nametab.class]
        if hexclr == nil then hexclr = "c8f60e" end
        tab.name = string.format("|cff%s%s|r", hexclr, name)
        tab.desc = string.format("Include/exclude %s in awarding points.", name)
        tab.get = function()
                    return memtab[n].onlist
                  end
        tab.set = function(v)
                    memtab[n].onlist = v
                    DKPmon:RefreshAwardUI()
                  end
        tab.order = string.byte(name, 1) -- the numeric code of the character name's first letter.
                                        -- Results in a list sorted by first letter of the player's names        
    end
    dewdrop:Open(self.GUI.awardUI.frame, 'children', dewOptions, 'point', "TOP", 'relativePoint', "TOP")
end

--[[
    DKPmon:PresentPoolSelectList()
    Description:
        Called when the user clicks in the text field for selecting which pool to award custom points to.
        Pops up a dewdrop dialog from which names can be selected
    Input:
        None
    Output:
        None
]]
function DKPmon:PresentPoolSelectList()
    -- Build the dewOptions table
    self:TableDelete(dewOptions)
    dewOptions = { type = "group", args = {} }
    dewOptions.args.header = {}
    dewOptions.args.header.type = "header"
    dewOptions.args.header.name = "List of pools"
    dewOptions.args.header.order = 1
    
    local pool, poolname
    for pool, poolname in pairs(self.poolnames) do
        local ind = ""..pool
        local p = pool
        dewOptions.args[ind] = {} 
        local tab = dewOptions.args[ind]
        tab.type = 'execute'
        tab.name = poolname
        tab.desc = string.format("Select pool %s", poolname)
        tab.func = function()
                        DKPmon.awards.selectedpool = p
                        dewdrop:Close(1)
                        DKPmon:RefreshAwardUI()
                   end
        tab.order = pool + 2        
     end
     dewdrop:Open(self.GUI.awardUI.frame, 'children', dewOptions, 'point', "TOP", 'relativePoint', "TOP")
end

--[[
    DKPmon:AwardBossPoints()
    Description:
        Called when the "Award Boss Points" button is pushed.
        Goes through the list of raid members & standby members and awards the boss points for them all.
    Input:
        None
    Output:
        None
]]
function DKPmon:AwardBossPoints()
    if self.awards.boss == nil then
        self:Print("ERROR: No boss selected")
        return
    end
    if StaticPopupDialogs["DKPMON_CONFIRM_AWARDBOSS"] == nil then
        self:CreateConfirmBossAwardDialog()
    end
    StaticPopup_Show("DKPMON_CONFIRM_AWARDBOSS")
end

function DKPmon:AwardCustomPoints()
    -- Validate input to the text box first
    local txt = self.GUI.awardUI.npoints:GetText()
    if tonumber(txt) == nil then
        self:Print("ERROR: Must enter a number in the box.")
        return
    end
    if StaticPopupDialogs["DKPMON_CONFIRM_AWARDCUSTOM"] == nil then
        self:CreateConfirmCustomAwardDialog()
    end
    StaticPopup_Show("DKPMON_CONFIRM_AWARDCUSTOM", tonumber(txt))
end


function DKPmon:AwardZeroSumPnts()
   -- Validate input to the text box first
    if StaticPopupDialogs["DKPMON_CONFIRM_AWARDZEROSUM"] == nil then
        self:CreateConfirmZerosumAwardDialog()
    end
    local npools = 0
    local msg = "Are you sure you want to award the following outstanding zero-sum points:\n"
    local pool, value
    for pool, value in pairs(self.db.realm.unawardedpts) do
        msg = string.format("%s%s -- %g\n", msg, self.poolnames[pool], value)
        npools = npools + 1
    end
    if npools == 0 then
        self:Print("No zero-sum points to award")
        return
    end
    StaticPopupDialogs["DKPMON_CONFIRM_AWARDZEROSUM"].text = msg
    StaticPopup_Show("DKPMON_CONFIRM_AWARDZEROSUM")
end

function DKPmon:BuildPlayersToAwardList()
    local list = {}
    local ndb, nraid, nstandby = 0, 0, 0
    local name, tab
    if self.GUI.awardUI.checkDB:GetChecked() == 1 then
        if self.awards.dblist ~= nil then
            for name, tab in pairs(self.awards.dblist) do
                if tab.onlist then
                    table.insert(list, tab.bidname)
                    ndb = ndb + 1
                end 
            end
        end
    end
    if self.GUI.awardUI.checkRaid:GetChecked() == 1 then
        for name, tab in pairs(self.awards.raid) do
            if tab.onlist then
                table.insert(list, tab.bidname)
                nraid = nraid + 1
            end
        end
    end
    if self.GUI.awardUI.checkStandby:GetChecked() == 1 then
        for name, tab in pairs(self.awards.standby) do
            if tab.onlist then
                table.insert(list, tab.bidname)
                nstandby = nstandby + 1
            end
        end
    end
    return list, nraid, nstandby, ndb
end

function DKPmon:AwardBossToList(list, bosstab)
    local index, pname
    local tab
    for index, pname in ipairs(list) do
        for _, tab in pairs(bosstab) do
            self:AwardPoints(pname, tab.pool, tab.value)
        end
    end
end

function DKPmon:ConfirmAwardBossPoints()
    if self.awards.boss == nil then
        self:Print("ERROR: No boss selected")
        return
    end
    local msg = string.format("Points awarded for killing %s", self.awards.boss)
    local tab
    local bosstab = self:GetBossTable(self.awards.boss)
     for _, tab in pairs(bosstab) do
        msg = string.format("%s; %g %s", msg, tab.value, self.poolnames[tab.pool])
    end
    local playersawarded, nraid, nstandby, ndb = self:BuildPlayersToAwardList()
    self:AwardBossToList(playersawarded, bosstab)
    if nraid > 0 then SendChatMessage(msg, "RAID") end
    if nstandby > 0 then SendChatMessage(msg, "GUILD") end
    if ndb > 0 then SendChatMessage(msg, "OFFICER") end
    if (nraid > 0) or (nstandby > 0) or (ndb > 0) then
        -- We actually awarded points, so we need to log that fact.
        local eventname = string.format("%s kill", self.awards.boss)
        for _,tab in pairs(bosstab) do
            self:LogDB_AddAwardEvent(eventname, tab.value, tab.pool, playersawarded)
        end
        self:Print("Boss award event logged")
    end
end

function DKPmon:AwardCustomToList(list, npnts, pool)
    local index, pname
    for index, pname in ipairs(list) do
        if npnts > 0 then
            self:AwardPoints(pname, pool, npnts)
        else
            self:LosePoints(pname, pool, -npnts)
        end
    end
end

function DKPmon:ConfirmAwardCustomPoints()
    local txt = self.GUI.awardUI.npoints:GetText()
    local npnts = tonumber(txt)
    local pool = self.awards.selectedpool
    if npnts == 0 then return end -- nothing to do
    local msg = npnts.." points awarded."

    local playersawarded, nraid, nstandby, ndb = self:BuildPlayersToAwardList()
    
    self:AwardCustomToList(playersawarded, npnts, pool)
    
    if nraid > 0 then SendChatMessage(msg, "RAID") end
    if nstandby > 0 then SendChatMessage(msg, "GUILD") end
    if ndb > 0 then SendChatMessage(msg, "OFFICER") end

    if (nraid > 0) or (nstandby > 0) or (ndb > 0) then
        -- We actually awarded points, so we need to log that fact.
        local eventname
        if npnts > 0 then
            eventname = "Custom points award"
        else
            eventname = "Custom points deduction"
        end
        -- Note: AddAwardEvent takes over the playersawarded table, do don't delete/use it anymore
        self:LogDB_AddAwardEvent(eventname, npnts, pool, playersawarded)
        self:Print("Custom award event logged")
    end
end

function DKPmon:ConfirmAwardZerosumPoints()
    local playersawarded, nraid, nstandby, ndb = self:BuildPlayersToAwardList()
    
    local totalplayers = nraid + nstandby + ndb
    if totalplayers == 0 then return end
    
    local eventname = "Zero-sum points award"
    local pool, points
    for pool, points in pairs(self.db.realm.unawardedpts) do
        pool = tonumber(pool)
        self:AwardCustomToList(playersawarded, points / totalplayers, pool)
        local msg = string.format("%g points awarded to %s", points / totalplayers, self.poolnames[pool])
        -- Tell people points were awarded
        if nraid > 0 then SendChatMessage(msg, "RAID") end
        if nstandby > 0 then SendChatMessage(msg, "GUILD") end
        if ndb > 0 then SendChatMessage(msg, "OFFICER") end
        -- Log the awarding
        self:LogDB_AddAwardEvent(eventname, points/totalplayers, pool, playersawarded)
    end
    self:TableDelete(self.db.realm.unawardedpts)
    self.db.realm.unawardedpnts = {}
end
