--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of DKPmon.

    DKPmon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DKPmon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DKPmon; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]
--[[

    Functions for building/displaying/manipulating the points awarding interface.

    DKPmon.GUI.awardUI.frame -- The points awarding interface frame
    DKPmon.GUI.awardUI.raidMembers -- FontString for displaying the number of raid members.
    DKPmon.GUI.awardUI.standbyMembers -- FontString for displaying the number of standby members.
    DKPmon.GUI.awardUI.bossName -- FontString for displaying the name of the boss selected.
    DKPmon.GUI.awardUI.playersSelected -- FontString for displaying the number of players selected.
    DKPmon.GUI.awardUI.checkRaid -- Checkbox for awarding to raid
    DKPmon.GUI.awardUI.checkStandby -- Checkbox for awarding to standby
    DKPmon.GUI.awardUI.checkDB -- Checkbox for awarding to database
    DKPmon.GUI.awardUI.npoints -- Editbox for entering how many custom points to award
    DKPmon.GUI.awardUI.poolselected -- Fontstring saying which pool was selecting for awarding custom points
    DKPmon.GUI.awardUI.zerosum -- Button for awarding zero sum points.

]]

local function CreateButton(parent, width, height, text, xoffs, yoffs)
    local b = CreateFrame("Button", nil, parent, "UIPanelButtonTemplate")
    b:SetWidth(width); b:SetHeight(height)
    b:SetText(text)
    if xoffs ~= nil and yoffs ~= nil then
        b:SetPoint("TOPLEFT", parent, "TOPLEFT", xoffs, yoffs)
    end
    return b
end

function DKPmon:CreateAwardFrame()
    --self:Print("Creating frame");
    local f, b, t
    DKPmon.GUI.bidUI = {}
    local frame = CreateFrame("Frame", "DKPmonAwardFrame", UIParent)
    frame:SetWidth(330)
    frame:SetHeight(310)
    frame:SetBackdrop( {
            bgFile = "Interface/Tooltips/UI-Tooltip-Background",
            edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
            tile = true, tileSize=16, edgeSize=16,
            insets = {left=4, right=4, top=4, bottom=4}
        } )
    frame:SetBackdropColor(0.16, 0.28, 0.61, 0.9) -- Blueish
    --frame:SetBackdropColor(0.29, 0.72, 0.70, 0.9) --Cyan
    --frame:SetBackdropColor(0.89, 0.15, 0.22, 0.9) -- Red
    frame:EnableMouse(true)
    frame:SetMovable(true)
    frame:SetPoint("CENTER", 0, 0)
    frame:CreateTitleRegion()
    local tR= frame:GetTitleRegion()
    tR:SetWidth(150); tR:SetHeight(20)
    tR:SetPoint("CENTER", frame, "TOP", 0, -10)

    -- Add the title to the top of the window
    t = frame:CreateFontString()
    t:SetAllPoints(tR)
    t:SetTextColor(1.0, 0.819, 0.0);
    t:SetWidth(140)
    t:SetFontObject(QuestTitleFont)
    t:SetText("Award Points")
    
    -- Add the "close window" button
    b = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
    b:SetPoint("CENTER", frame, "TOPRIGHT", -16, -16)
    b:Show()
    
    -- The raid contents line
    t = frame:CreateFontString()
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 10,-30)
    t:SetFontObject(GameFontWhite); t:SetWidth(180)
    t:SetText("Raid contains 0 members")
    self.GUI.awardUI.raidMembers = t
    f = CreateFrame("Frame", nil, frame)
    f:SetWidth(180);  f:SetHeight(30)
    f:SetPoint("TOPLEFT", frame, "TOPLEFT", 10,-30)
    f:EnableMouse(true)
    f:SetScript("OnMouseDown", function() DKPmon:PresentRaidList() end)
    b = CreateButton(frame, 125, 22, "Query Raid", 190, -30)
    b:SetScript("OnClick", function() 
                            DKPmon:SendClaimLeader(); 
                            DKPmon:TableDelete(DKPmon.awards.raid); 
                            DKPmon.awards.raid = {}
                            DKPmon:RefreshAwardUI()
                            DKPmon:SendQueryRaid() 
                           end)
    
    -- The standby contents line
    t = frame:CreateFontString()
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 10,-60)
    t:SetFontObject(GameFontWhite); t:SetWidth(180)
    t:SetText("0 people on standby")
    self.GUI.awardUI.standbyMembers = t
    f = CreateFrame("Frame", nil, frame)
    f:SetWidth(180);  f:SetHeight(30)
    f:SetPoint("TOPLEFT", frame, "TOPLEFT", 10,-60)
    f:EnableMouse(true)
    f:SetScript("OnMouseDown", function() DKPmon:PresentStandbyList() end)
    b = CreateButton(frame, 125, 22, "Query Standby", 190, -60)
    b:SetScript("OnClick", function() 
                            DKPmon:SendClaimLeader();
                            DKPmon:TableDelete(DKPmon.awards.standby); 
                            DKPmon.awards.standby = {}
                            DKPmon:RefreshAwardUI()                            
                            DKPmon:SendQueryStandby() 
                           end)
    
    
    -- Selecting names from the database line
    f = CreateFrame("Frame", nil, frame)
    f:SetWidth(310); f:SetHeight(30)
    f:SetPoint("TOPLEFT", frame, "TOPLEFT", 10, -90)
    f:EnableMouse(true)
    f:SetScript("OnMouseDown", function() DKPmon:PresentDBNameSelectList() end)
    t = f:CreateFontString()
    t:SetFontObject(GameFontWhite); t:SetWidth(310)
    t:SetText("0 players selected from the database")
    t:SetPoint("TOPLEFT", f, "TOPLEFT", 0, 0)
    DKPmon.GUI.awardUI.playersSelected  = t
    
    -- Checkboxes for which lists to use in awarding points.
    t = frame:CreateFontString()
    t:SetFontObject(GameFontWhite); t:SetWidth(70)
    t:SetText("Award to --")
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 10, -140)
    
    t = frame:CreateFontString()
    t:SetFontObject(GameFontWhite); t:SetWidth(30)
    t:SetText("Raid:")
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 85, -140)
    b = CreateFrame("CheckButton", nil, frame, "UICheckButtonTemplate")
    b:SetWidth(30); b:SetHeight(30)
    b:SetPoint("TOPLEFT", frame, "TOPLEFT", 115, -130)
    self.GUI.awardUI.checkRaid = b
    
    t = frame:CreateFontString()
    t:SetFontObject(GameFontWhite); t:SetWidth(60)
    t:SetText("Standby:")
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 155, -140)
    b = CreateFrame("CheckButton", nil, frame, "UICheckButtonTemplate")
    b:SetWidth(30); b:SetHeight(30)
    b:SetPoint("TOPLEFT", frame, "TOPLEFT", 215, -130)
    self.GUI.awardUI.checkStandby = b

    t = frame:CreateFontString()
    t:SetFontObject(GameFontWhite); t:SetWidth(30)
    t:SetText("DB:")
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 260, -140)
    b = CreateFrame("CheckButton", nil, frame, "UICheckButtonTemplate")
    b:SetWidth(30); b:SetHeight(30)
    b:SetPoint("TOPLEFT", frame, "TOPLEFT", 290, -130)
    self.GUI.awardUI.checkDB = b
    
    -- Line for awarding custom points
    t = frame:CreateFontString()
    t:SetFontObject(GameFontWhite); t:SetWidth(65)
    t:SetText("#points:")
    t:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 30, 120)
    f = CreateFrame("EditBox", nil, frame, "InputBoxTemplate")
    f:SetWidth(50);  f:SetHeight(30)
    f:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 90, 130)
    f:SetNumber(0)
    f:SetAutoFocus(false)
    self.GUI.awardUI.npoints = f
    
    -- Clicker for the pool choice
    f = CreateFrame("Frame", nil, frame)
    f:SetWidth(120); f:SetHeight(25)
    f:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 40, 140)
    f:EnableMouse(true)
    f:SetScript("OnMouseDown", function() DKPmon:PresentPoolSelectList() end)
    t = f:CreateFontString()
    t:SetFontObject(GameFontWhite); t:SetWidth(90)
    t:SetText("")
    t:SetPoint("TOPLEFT", f, "TOPLEFT", 0, 0)
    DKPmon.GUI.awardUI.poolselected  = t
    
    b = CreateButton(frame, 125, 22, "Award Custom")
    b:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 190, 130)
    b:SetScript("OnClick", function() DKPmon:AwardCustomPoints() end)
    
    -- Line for awarding boss points
    f = CreateFrame("Frame", nil, frame)
    f:SetWidth(180);  f:SetHeight(30)
    f:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 10,90)
    f:EnableMouse(true)
    f:SetScript("OnMouseDown", function() DKPmon:PresentBossList() end)
    t = f:CreateFontString()
    t:SetPoint("TOPLEFT", f, "TOPLEFT", 0, 0)
    t:SetFontObject(GameFontWhite); t:SetWidth(180)
    t:SetText("No boss selected")
    self.GUI.awardUI.bossName = t
    b = CreateButton(frame, 125, 22, "Award Boss Points") --, 190, -110)
    b:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 190, 90)
    b:SetScript("OnClick", function() DKPmon:AwardBossPoints() end)

    -- Line for awarding zero-sum points
    b = CreateButton(frame, 250, 22, "Award outstanding zero-sum points")
    b:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 35, 60)
    b:SetScript("OnClick", function() DKPmon:AwardZeroSumPnts() end)
    self.GUI.awardUI.zerosum = b
    
    -- Button for broadcasting points
    b = CreateButton(frame, 250, 22, "Broadcast Points Database") --, 35, -150)
    b:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 35, 30)
    b:SetScript("OnClick", function() DKPmon:BroadcastPointsDB() end)
    
 --[[   
    -- Draw the measuring stick
    local x = 10
    while x ~= 250 do
        t = frame:CreateFontString()
        t:SetPoint("TOPLEFT", frame, "TOPLEFT", x,-20)
        t:SetFontObject(GameFontWhite); t:SetWidth(5)
        t:SetJustifyH("LEFT")
        if math.mod(x, 20) == 0 then
            t:SetText("|")
        else
            t:SetText(".")
        end
        x = x + 10
    end
]]

    -- Create the tablet that'll contain the bidder list when the user clicks
    local dewdrop = AceLibrary("Dewdrop-2.0")
    dewdrop:Register(frame, { "dontHook", true })
    self.GUI.awardUI.frame = frame
    frame:Hide()
end

function DKPmon:ShowAwardUI()
    if self.GUI.awardUI.frame == nil then self:CreateAwardFrame() end
    self:RefreshAwardUI()
    self.GUI.awardUI.frame:Show()
end

function DKPmon:RefreshAwardUI()
    if self.GUI.awardUI.frame == nil then return end
    
    local n,v
    local nraiders, nstandby, ndbselected =0,0,0
    nraiders = self:CountRaidMembers()
    nstandby = self:CountStandbyMembers()
    ndbselected = self:CountDBSelected()
    self.GUI.awardUI.raidMembers:SetText(string.format("Raid contains %d raid members", nraiders))
    self.GUI.awardUI.standbyMembers:SetText(string.format("%d players on standby", nstandby))
    self.GUI.awardUI.playersSelected:SetText(string.format("%d players selected from the database", ndbselected))
    self.GUI.awardUI.poolselected:SetText(string.format("Pool: %s", self.poolnames[self.awards.selectedpool]))
    --self.GUI.awardUI.npoints:SetText("0")
    if self.awards.boss == nil then
        self.GUI.awardUI.bossName:SetText("No boss selected")
    else
        self.GUI.awardUI.bossName:SetText(self.awards.boss)
    end
    if self.db.realm.usezerosum then
        self.GUI.awardUI.zerosum:Enable()
    else
        self.GUI.awardUI.zerosum:Disable()
    end
end

function DKPmon:CreateConfirmBossAwardDialog()
    StaticPopupDialogs["DKPMON_CONFIRM_AWARDBOSS"] = {
        text = "Are you sure you want to award boss points?",
        button1 = "Yes",
        button2 = "No",
        OnAccept = function()
            DKPmon:ConfirmAwardBossPoints()
        end,
        OnCancel = function() end,
        sound = "levelup2",
        timeout = 10,
        whiledead = 1,
        hideOnEscape = 1
    }
end

function DKPmon:CreateConfirmCustomAwardDialog()
    StaticPopupDialogs["DKPMON_CONFIRM_AWARDCUSTOM"] = {
        text = "Are you sure you want to award %g custom points?",
        button1 = "Yes",
        button2 = "No",
        OnAccept = function()
            DKPmon:ConfirmAwardCustomPoints()
        end,
        OnCancel = function() end,
        sound = "levelup2",
        timeout = 10,
        whiledead = 1,
        hideOnEscape = 1
    }
end

function DKPmon:CreateConfirmZerosumAwardDialog()
    StaticPopupDialogs["DKPMON_CONFIRM_AWARDZEROSUM"] = {
        text = "",
        button1 = "Yes",
        button2 = "No",
        OnAccept = function()
            DKPmon:ConfirmAwardZerosumPoints()
        end,
        OnCancel = function() end,
        sound = "levelup2",
        timeout = 10,
        whiledead = 1,
        hideOnEscape = 1
    }
end