﻿--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of DKPmon.

    DKPmon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DKPmon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DKPmon; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]
--[[
    Functions for inter-addon communication with DKPmon
]]

local Comm_Version = 1.3

--[[
    DKPmon:SendMessage
    Description:
        Combines sending to "GROUP" and "WHISPER" so that we receive our own messages.
    Input:
        p0...p15: Arguments to self:SendCommMessage("GROUP", ...)
    Output:
        None
]]
function DKPmon:SendMessage(p0, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15)
    local sent = self:SendCommMessage("GROUP", Comm_Version, p0, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15)
    if sent == true then
        DKPmon:OnCommReceive(self.DKPmonCommPrefix, UnitName("player"), "GROUP", Comm_Version, p0, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15)
        Bidder:OnCommReceive(self.DKPmonCommPrefix, UnitName("player"), "GROUP", Comm_Version, p0, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15)
        --self:SendCommMessage("WHISPER", UnitName("player"), Comm_Version, p0, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15)
    end
end

--[[
    DKPmon:SendOpenBidding()
    Description:
        Opens bidding on the current list of bidable items
    Input:
        None
    Output:
        None
]]
function DKPmon:SendOpenBidding()
    SendChatMessage("Bidding is now open on:", "RAID")
    local i=0
    local s = {}
    while i ~= self.bidding.nitems do
        local tab = self.bidding[i]
        SendChatMessage("("..tab.cost.." "..self.poolnames[tab.pool].." pts):"..tab.link, "RAID")
        -- Send: "<pool#>:<cost>:<colour>:<itemname>"
        s[i] = string.format("%d:%g:%s:%s", tab.pool, tab.cost, tab.colour, tab.name) --tab.pool..":"..tab.cost..":"..tab.colour..":"..tab.name
        i = i + 1
    end
    self:SendMessage("B", s[0], s[1], s[2], s[3], s[4], s[5], s[6], s[7], s[8], s[9])
    self:TableDelete(s)
end

--[[
    DKPmon:SendCloseBidding()
    Description:
        Sends the broadcast message that closes the bidding process
    Input:
        None
    Output:
        None
]]
function DKPmon:SendCloseBidding()
    self:SendMessage("C")
end

--[[
    DKPmon:SendClaimLeader()
    Description:
        Sends a message to the other DKPmon mods in the raid to tell them that you're
        taking over as the DKPmon-leader (ie: we'll reply to queries, etc)
    Input:
        None
    Output:
        None
]]
function DKPmon:SendClaimLeader()
    self.amLeader = true
    self:SendCommMessage("GROUP", Comm_Version, "L") -- We actually don't want to send this one to ourselves.
end

--[[
    DKPmon:SendQueryRaid()
    Description:
        Sends a message requesting that all Bidder addons in raid reply with their bidnames.
        Used for building the list of people in the raid to award points to.
    Input:
        None
    Output:
        None
]]
function DKPmon:SendQueryRaid()
    self:SendMessage("QR")
    --self:Print("SendQueryRaid")
end

--[[
    DKPmon:SendQueryStandby()
    Description:
        Sends a message requesting that all Bidder addons in guild reply with their bidnames.
        Used for building the list of people on standby to award points to
    Input:
        None
    Output:
        None
]]
function DKPmon:SendQueryStandby()
    self:SendCommMessage("GUILD", Comm_Version, "QS")
    --self:Print("SendQueryStandby")
end

--[[
    DKPmon:SendBroadcastPointsDB()
    Description:
        Broadcast our points database to the hidden raid channel
    Input:
        None
    Output:
        None
]]
function DKPmon:SendBroadcastPointsDB()
    local name, ptstab
    local toSend = ""
    for name, ptstab in pairs(self.db.realm.points ) do
        local msg = name..";"
        if ptstab.info.class ~= nil then
            msg = msg..ptstab.info.class.."~"
        else
            msg = msg.."UNKNOWNN".."~"
        end
        --local msg = name..";"..ptstab.info.class.."~"
        local pool, pooltab
        for pool, pooltab in pairs(ptstab.points) do
            local p = tonumber(pool)
            msg = string.format("%s%d(%g:%g:%g)", msg, p, pooltab.earned, pooltab.spent, pooltab.lost)
        end
        toSend = string.format("%s%s!", toSend, msg)
    end
    self:SendCommMessage("GROUP", Comm_Version, "BRC", self.db.realm.password, toSend) -- Don't broadcast to ourself
    --self:SendMessage("BRC", toSend)
    self:Print("Points database has been broadcast")
end

--[[
    DKPmon:SendSyncRequest()
    Description:
        Broadcasts a request to sync databases
    Input:
        None
    Output:
        None
]]
function DKPmon:SendSyncRequest()
    self:Print("Sending sync request")
    self:SendMessage("SYNC") -- We want this one to broadcast to ourselves.
end

--[[
    DKPmon:SendBidderInstallCheck()
    Description:
        Broadcasts a request for all bidders to respond
        Used to initiate a check for who in the raid doesn't have bidder
    Input:
        None
    Output:
        None
]]
function DKPmon:SendBidderInstallCheck()
    self:TableDelete(self.raidList)
    self.raidList = self:BuildRaidmemberList()
    self:SendMessage("QI")
end

--[[
    DKPmon:CommRecvBroadcastPointsDB(msg)
    Description:
        Receive a points database update via broadcast
    Input:
        None
    Output:
        None
]]
function DKPmon:CommRecvBroadcastPointsDB(msg)
    -- Pull the name from the message
    local _, _, name = string.find(msg, "(.+);")
    local _, _, class = string.find(msg, ";(.+)~")
    local p, e, s, l
    for p,e,s,l in string.gmatch(msg, "([0-9])%(([-]?[%d]*%.?[%d]+):([-]?[%d]*%.?[%d]+):([-]?[%d]*%.?[%d]+)%)") do
        local pool, earned, spent, lost = tonumber(p), tonumber(e), tonumber(s), tonumber(l)
        self:UpdatePoints(name, pool, earned, spent, lost)
    end
    if class ~= "UNKNOWN" then
        self.db.realm.points[name].info.class = class
    end
end

--[[
    DKPmon:CommRecvQueryPoints(sender, name, console)
    Description:
        Receive a query for points for bidder "name"
    Input:
        sender: The name of the person who sent the query
        name: The bidder name to query
        console: boolean; true = reply via whisper
    Output:
        None
]]
function DKPmon:CommRecvQueryPoints(sender, name, console)
    local pnts = {}
    -- Build the table of how many points the name has.
    local p, v
    for p, _ in pairs(self.poolnames) do
        pnts[p] = self:GetPlayerPoints(name, p)
    end
    if console == true then
        -- Reply via whisper
        local reply = "Your points are:"
        for p, v in pairs(pnts) do
            reply = string.format("%s %s = %g;", reply, self.poolnames[p], v) --reply.."  "..self.poolnames[p].." = "..v;
        end
        --self:SendCommMessage("WHISPER", sender, Comm_Version, "RQC", reply)
        self:SendMessage("RQC", sender, reply)
    else
        local i=0
        local msg = ":"
        while pnts[i] ~= nil do
            msg = string.format("%s%g:", msg, pnts[i])
            i = i + 1
        end
        --self:Print(string.format("Sending to %s: %s %s %s %s %s %s %s %s %s %s", name, 
                        --pnts[0], pnts[1], pnts[2], pnts[3], pnts[4], pnts[5], pnts[6], pnts[7], pnts[8], pnts[9] ))
        --self:SendCommMessage("WHISPER", sender, Comm_Version, "RQP", msg)
        self:SendMessage("RQP", sender, msg)
    end
    self:TableDelete(pnts)
end

--[[
    DKPmon:CommRecvBid(sender, bidname, bids)
    Description:
        Receive a bid placement from bidname
    Input:
        sender: The name of the person who sent the query
        senderclass: The english class name of the sender (ex: "WARRIOR")
        bidname: The name the person's bidding under
        bidclass: The class of the bidname
        bids: String. Each character is a 'Y' or 'N' saying whether they're bidding on the corresponding item
    Output:
        None
]]
function DKPmon:CommRecvBid(sender, senderclass, bidname, bids)
    if self.bidding.bidstate ~= 1 then 
        -- Bidding has closed, they're too late.
        --self:SendCommMessage("WHISPER", sender, Comm_Version, "ACKB", "Sorry, bidding is not open.")
        self:SendMessage("ACKB", sender, "Sorry, bidding is not open.")
        return
    end
    -- Okay, parse the list of bids and place a bid for each one.
    local nbids = 0
    local itemID = 0
    local msg = string.format("%s is now bidding on item numbers:", bidname)
    for bid in string.gmatch(bids, "([YN])") do
        local added = self:PlaceBid(itemID, bidname, sender, senderclass, bid)
        if added then
            if nbids == 0 then msg = string.format("%s %d", msg, itemID)
            else msg = string.format("%s, %d", msg, itemID)
            end
            nbids = nbids + 1
        end
        itemID = itemID + 1
    end
    if nbids == 0 then msg = string.format("Thank you, %s is now bidding on zero items.", bidname)
    else msg = string.format("%s, thank you.", msg)
    end
    -- Send a message to the sender confirming the bids were placed.
    --self:SendCommMessage("WHISPER", sender, Comm_Version, "ACKB", msg)
    self:SendMessage("ACKB", sender, msg)
end

--[[
    DKPmon:CommRecvRaidQueryReply(bidname, sender)
    Description:
        Receive a reply to the query raid command
    Input:
        bidname: The bidname of the person who replied.
        sender: The name of the person who sent the reply
    Output:
        None
]]
function DKPmon:CommRecvRaidQueryReply(bidname, sender, senderclass)
    self.awards.raid[sender] = { onlist = true, bidname = bidname, senderclass = senderclass }
    self:RefreshAwardUI()
end

--[[
    DKPmon:CommRecvStandbyQueryReply(bidname, sender)
    Description:
        Receive a reply to the query standby command
    Input:
        bidname: The bidname of the person who replied.
        sender: The name of the person who sent the reply
    Output:
        None
]]
function DKPmon:CommRecvStandbyQueryReply(bidname, sender, senderclass)
    --self:Print("Got standby")
    self.awards.standby[sender] = { onlist = true, bidname = bidname, senderclass = senderclass }
    self:RefreshAwardUI()
end

--[[
    AceComm-2.0 callback
]]
function DKPmon:OnCommReceive(prefix, sender, distribution, ver, a0, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11)
    --self:Print("Got message with prefix = "..prefix.." from "..sender.." on "..distribution.." of type "..a0)
    if prefix == self.DKPmonCommPrefix then
        -- Message from non-compatible DKPmon
        if ver ~= Comm_Version then return end
        -- Message from DKPmon
        --[[
            "L" -- Another DKPmon is claiming leader.
            "BRC" -- Broadcast data being sent down the pipe
            "SYNC" -- A request to sync databases
            "ACKBRC" -- Sender received your broadcast
        ]]
        if a0 == "L" then
            -- Another DKPmon is claiming leader status. Set ourselves to not the leader.
            self.amLeader = false
            -- Also make a note of when the message was received
            self.lastLeaderClaim = GetTime()
            return
        end
        if a0 == "BRC" then
            -- a1 is the broadcast password. It must match our password to be accepted
            if not (a1 == self.db.realm.password) then
                return -- Just ignore this message
            end
            -- a2 is the broadcast data
            self:Print("Received db broadcast from "..sender)
            local part
            for part in string.gmatch(a2, "([%w%(%)%.%-%:%;%~]+)") do
                --self:Print("dbpart = "..part)
                self:CommRecvBroadcastPointsDB(part)
            end
            -- Send confirmation of receiving the broadcast
            --self:SendCommMessage("WHISPER", sender, Comm_Version, "ACKBRC")
            self:SendMessage("ACKBRC", sender)
            return
        end
        if a0 == "SYNC" then
            self:Print("Received db sync request from "..sender)
            self:SendBroadcastPointsDB()
            return
        end
        if a0 == "ACKBRC" then
            if (a1 ~= UnitName("player")) then return end
            self:Print(string.format("%s received db broadcast", sender))
            return
        end
        return
    end
    if prefix == self.BidderCommPrefix then
        -- Message from non-compatible Bidder
        if ver ~= Comm_Version then return end
        --self:Print("prefix = "..a0)
        -- Message from Bidder
        --[[
            Okay, a0 contains the message command.
            "QC" -- message is a query for points from a console command
            "QP" -- message is a query for points for the purposes of bidding
            "RB" -- message is placing bids on items
            "RQR" -- message is a reply to a raid query
            "RQS" -- message is a reply to a standby query
            "RQI" -- reply to bidder installation check
            
            Note: QC is the only command that is sent from Bidder via broadcast, all the rest are whispers.
        ]]
        --self:Print(a0)
        if a0 == "QC" then
            -- a1 = name to query the points of
            if self.amLeader == false then return end
            -- a1 = Bidding name to query the points under
            self:CommRecvQueryPoints(sender, a1, true)
            return
        end
        if a0 == "QP" then
            --if self.amLeader == false then return end
            -- a1 = Person the query is aimed at
            -- a2 = Bidding name to query the points under
            if (a1 ~= UnitName("player")) then return end
            self:CommRecvQueryPoints(sender, a2, false)
            return
        end 
        if a0 == "RB" then
            --if self.amLeader == false then return end
            -- Message: "RB", bidname, senderclass, biddingclass, bidstring
            -- a1 = Person the query is aimed at
            -- a2 = bidding name
            -- a3 = number indicating the class of the sender
            -- a4 = number indicating the class of the bidding name
            -- a5 = list of items bid on
            if (a1 ~= UnitName("player")) then return end
            if self.bidderClasses[a2] == nil then
                self.bidderClasses[a2] = Bidder.classInfo.numToClass[a4]
            end
            self:CommRecvBid(sender, Bidder.classInfo.numToClass[a3], a2, a5)
            return
        end
        if a0 == "RQR" then
            --if self.amLeader == false then return end
            -- a1 = Person the query is aimed at
            -- a2 - number: classID of the sender
            -- a3 - bidding name
            -- a4 - number: classID of the bidder
            if (a1 ~= UnitName("player")) then return end
            if self.bidderClasses[a3] == nil then
                self.bidderClasses[a3] = Bidder.classInfo.numToClass[a4]
            end
            self:CommRecvRaidQueryReply(a3, sender, Bidder.classInfo.numToClass[a2])
            return
        end
        if a0 == "RQS" then
            --if self.amLeader == false then return end
            -- a1 = Person the query is aimed at
            -- a2 - number: classID of the sender
            -- a3 - bidding name
            -- a4 - number: classID of the bidder
            if (a1 ~= UnitName("player")) then return end
            if self.bidderClasses[a3] == nil then
                self.bidderClasses[a3] = Bidder.classInfo.numToClass[a4]
            end
            self:CommRecvStandbyQueryReply(a3, sender, Bidder.classInfo.numToClass[a2])
            return
        end
        if a0 == "RQI" then
            if (a1 ~= UnitName("player")) then return end
            self.raidList[sender] = 1
            return
        end
        return
    end
end