﻿--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of DKPmon.

    DKPmon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DKPmon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DKPmon; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]
--[[
    User-specific boss point values.
    
    - Bosses are stored in tables indexed by the instance/zone in 
     which the boss is found.
        - Note: The instance name should be spelled right. :-)
    
    - Each boss entry is a table indexed by the boss's name and
    contains a list of (value,poolindex) pairs.
    - Note that the boss' name -should- be identical to how you'd
    see it in the game.
    
    ex: ["Molten Core"] = {
                ["Lucifron"] = { [0] = { value = 10, pool = 0 },
                                 [1] = { value = 20, pool = 1} }
        }
       -- This means that the "Molten Core" boss Lucifron is worth 
        10 pool 0 points and 20 pool 1 points when killed.
]]

--[[
    Reprisal point pools (see Custom\poolnames.lua):
     0 - MC/Ony, 1 - BWL, 2 - AQ40, 3 - Naxx
]]

DKPmon.bossvalues = {
    -- Molten Core bosses
    ["The Molten Core"] = {
        ["Lucifron"] = { [0]={value=6, pool=0} },
        ["Magmadar"] = { [0]={value=6, pool=0} },
        ["Gahennas"] = { [0]={value=8, pool=0} },
        ["Garr"]     = { [0]={value=10, pool=0} },
        ["Baron Geddon"] = { [0]={value=10, pool=0} },
        ["Shazzrah"] = { [0]={value=10, pool=0} },
        ["Harbinger Sulfuron"] = { [0]={value=10, pool=0} },
        ["Golemagg"] = { [0]={value=10, pool=0} },
        ["Major Domo"] = { [0]={value=14, pool=0} },
        ["Ragnaros"] = { [0]={value=20, pool=0} }
    },
    ["Onyxia's Lair"] = {
        ["Onyxia"] = { [0]={value=20, pool=0} }
    },
    
    -- Blackwing Lair bosses
    ["Blackwing Lair"] = {
        ["Razorgore"] = { [0]={value=22, pool=1} },
        ["Vael"] = { [0]={value=25, pool=1} },
        ["Broodlord"] = { [0]={value=25, pool=1} },
        ["Firemaw"] = { [0]={value=25, pool=1} },
        ["Ebonroc"] = { [0]={value=20, pool=1} },
        ["Flamegor"] = { [0]={value=20, pool=1} },
        ["Chromaggus"] = { [0]={value=28, pool=1} },
        ["Nefarian"] = { [0]={value=35, pool=1} }
    },
    
    -- AQ40 bosses
    ["Ahn'Qiraj"] = {
        ["Skeram"] = { [0]={value=6, pool=2} },
        ["Bug Family"] = { [0]={value=8, pool=2} },
        ["Sartura"] = { [0]={value=10, pool=2} },
        ["Fankriss"] = { [0]={value=10, pool=2} },
        ["Huhuran"] = { [0]={value=14, pool=2} },
        ["Viscidus"] = { [0]={value=10, pool=2} },
        ["Twin Emperors"] = { [0]={value=16, pool=2} },
        ["Ouro"] = { [0]={value=16, pool=2} },
        ["C'thun"] = { [0]={value=20, pool=2} }
    },
    
    -- Naxx bosses
    ["Naxxramas"] = {
        ["Spider Wing"] = {
            ["Anub'Rekhan"] = { [0]={value=25, pool=3} },
            ["Grand Widow Faerlina"] = { [0]={value=30, pool=3} },
            ["Maexxna"] = { [0]={value=40, pool=3} }
        },
        ["Deathknight Wing"] = {
            ["Instructor Razuvious"] = { [0]={value=25, pool=3} },
            ["Gothik the Harvester"] = { [0]={value=30, pool=3} },
            ["The Four Horsemen"] = { [0]={value=40, pool=3} }
        },
        ["Plague Wing"] = {
            ["Noth the Plaguebringer"] = { [0]={value=25, pool=3} },
            ["Heigan the Unclean"] = { [0]={value=30, pool=3} },
            ["Loatheb"] = { [0]={value=40, pool=3} }
        },
        ["Abomination Wing"] = {
            ["Patchwerk"] = { [0]={value=20, pool=3} },
            ["Grobbulus"] = { [0]={value=25, pool=3} },
            ["Gluth"] = { [0]={value=30, pool=3} },
            ["Thaddius"] = { [0]={value=40, pool=3} }
        },
        ["Frostwyrm Lair"] = {
            ["Sapphiron"] = { [0]={value=50, pool=3} },
            ["Kel'Thuzad"] = { [0]={value=50, pool=3} }
        }
    }
}
