--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of DKPmon.

    DKPmon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DKPmon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DKPmon; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]

--[[
   Log format:
   
      self.db.realm.log -- Table, indexed by date, as a string, in month/day/year format
      self.db.realm.log[<date>] -- Table, number indexed starting at 1. Each element has this format:
         Common fields:
            {
               time -- String, the time the event was added to the log. "%X" format
               name -- String, the name given to the event
               type -- String, the type of the event ['award', 'item']
               points -- Number: the number of points awarded/deducted
               pool   -- Number: the pool the points were awarded/deducted from
            }
         .type = 'award' fields  (a points awarding event)
            {
               players  -- Table: the list of players awarded the points. Number indexed, starting at 1
            }
         .type = 'item' fields  (a points deduction event for winning an item)
            {
               winner -- String: The name of the player who was given the item
            }
]]

--[[
    DKPmon:LogDB_AddAwardEvent(eventname, npnts, pool, playerlist)
    Description:
        Adds a point-awarding event to the log
    Input:
        eventname: The title to give the event
        npnts: The number of points awarded
        pool: The pool in which the points were awarded
        playerlist: Table/array; [i] = string -- name of a player who received points
    Output:
        None
]]
function DKPmon:LogDB_AddAwardEvent(eventname, npnts, pool, playerlist)
   local currDay = date("%m/%d/%y")
   -- Check if we have a table, yet, for today
   if self.db.realm.log[currDay] == nil then
      self.db.realm.log[currDay] = {}
   end
   local tab = {}
   tab.time = date("%X")
   tab.name = eventname
   tab.type = 'award'
   tab.points = npnts
   tab.pool = pool
   tab.players = playerlist
   table.insert(self.db.realm.log[currDay], tab) -- plop the event onto the end of the list
end

--[[
    DKPmon:LogDB_AddItemEvent(itemname, npnts, pool, winner)
    Description:
        Adds an item-awarded event to the log
    Input:
        itemname: The name of the item won
        npnts: The cost of the item
        pool: The pool the item cost came from
        winner: String -- the name of the winner
    Output:
        None
]]
function DKPmon:LogDB_AddItemEvent(itemname, npnts, pool, winner)
   local currDay = date("%m/%d/%y")
   -- Check if we have a table, yet, for today
   if self.db.realm.log[currDay] == nil then
      self.db.realm.log[currDay] = {}
   end
   local tab = {}
   tab.time = date("%X")
   tab.name = itemname
   tab.type = 'item'
   tab.points = npnts
   tab.pool = pool
   tab.winner = winner
   table.insert(self.db.realm.log[currDay], tab) -- plop the event onto the end of the list
end