﻿--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of DKPmon.

    DKPmon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DKPmon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DKPmon; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]

DKPmon = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceComm-2.0", "Metrognome-2.0", "AceEvent-2.0")

--[[
    Initialize the database. 
    Database entries are:
        points: Table containing a table for each player's points.
          -- Indexed by the player's name.
          -- Each player-table is a list indexed by pool# of tables.
          -- A nil for a pool# index means that player has no standing, yet, in that pool
          -- Each table contains:
            earned: Total points earned in the pool
            spent:  Total points spent in the pool
            lost:   Total points lost from the pool (due to decay, etc)
]]

local DKPmonDBver = 1.2

DKPmon:RegisterDB("DKPmonDB")
DKPmon:RegisterDefaults('realm', {
    points = {},
    log = {},
    unawardedpts = {},
    dbversion = 0,
    autoloot = false,
    usezerosum = false,
    password = ""
    } )

function DKPmon:OnInitialize()
    -- Called when the addon is loaded

    -- Ensure the database is up to date.
    if self.db.realm.dbversion < DKPmonDBver and not (self.db.realm.dbversion == 0) then
        -- Database is from a previous version of DKPmon
        -- Need to migrate it over.
        self:Print("Database version changed. Migrating to new format.")
        self:MigrateDB()
    end
    self.db.realm.dbversion = DKPmonDBver

    self.GUI = {} 
    self.GUI.bidUI = {} 
    self.GUI.dumpUI = {} 

    -- Message prefix to use in inter-addon communication
    self.BidderCommPrefix = "Bidder"
    self.DKPmonCommPrefix = "DKPmon"
    self.CommPrefix = self.DKPmonCommPrefix

    if BidderDKPStub ~= nil then
        BidderDKPStub:DeregisterComm()
    end
    self:SetCommPrefix(self.CommPrefix)

    self:InitBiddingFields()
    self:InitAwardFields()
    
    -- Table indexed by the name of members in the raid.
    -- each entry is a number
    -- This table is used for querying to see who in the raid doesn't
    -- have Bidder installed.
    self.raidList = {}
    
    -- Table indexed by the names of bidders we've received this session
    self.bidderClasses = {} 
    
    -- Register the metronome
    self:RegisterMetro("DKPmonClose", function() DKPmon:StartCloseBiddingTimer() end)
    self:RegisterMetro("DKPmonBidderCheck", function() DKPmon:PrintNoBidderInstalledList() end)
    self:RegisterMetro("DKPmonLeader", function() DKPmon:CheckLeader() end)
 
end

function DKPmon:OnEnable()
    -- Called when the addon is enabled.
    self:Print("Enabled")
    -- Claim our message prefix from the Bidder stub hack
    -- Register to receieve messages from DKPmon and Bidder over GROUP
    self:RegisterComm(self.DKPmonCommPrefix, "GROUP")
    self:RegisterComm(self.DKPmonCommPrefix, "WHISPER")
    self:RegisterComm(self.BidderCommPrefix, "GROUP")
    self:RegisterComm(self.BidderCommPrefix, "WHISPER")
    self:RegisterComm(self.BidderCommPrefix, "GUILD")
    
    self.amLeader = false
    self.lastLeaderClaim = GetTime()
    
    self:ChangeMetroRate("DKPmonLeader", 17) -- Call every 17 seconds
    self:StartMetro("DKPmonLeader")
    
    self:RegisterEvent("LOOT_OPENED", "OnLootEvent")
end

function DKPmon:OnDisable()
    -- Called when the addon is disabled.
    self:Print("Disabled")
end

--[[ AceDB-2.0 Methods ]]
function DKPmon:OnProfileDisable()
    -- Called when the user changes profiles (before change)
end

function DKPmon:OnProfileEnable()
    -- Called when the user changes profiles (after change)
end

--[[ Function for checking the whether the raid has a DKPmon leader set ]]
function DKPmon:CheckLeader()
    if GetNumRaidMembers() == 0 then
        -- Not in a raid, ignore.
        return
    end
    if self.amLeader then
        -- We're the leader, send a message that we're the leader to the raid.
        self:SendClaimLeader()
    else
        -- We're not the leader. If it's been more than 45 seconds since we last had a DKPmon
        -- claim leader, then claim leader for ourself.
        local currTime = GetTime()
        if (currTime - self.lastLeaderClaim) > 45.0 then
            self.amLeader = true
            self:SendClaimLeader()
        end
    end
end